@@ -205,6 +205,7 @@ log_prfn which_prep_geom_fn(const std::string op) {
 	Rcpp::stop("wrong value for op"); // unlikely to happen unless user wants to #nocov
 }
 
+/*
 Rcpp::LogicalVector get_dense(std::vector<size_t> items, int length) {
 	Rcpp::LogicalVector rowi(length);
 	for (int j = 0; j < length; j++)
@@ -213,10 +214,11 @@ Rcpp::LogicalVector get_dense(std::vector<size_t> items, int length) {
 		rowi(items[j] - 1) = true; // items is 1-based
 	return rowi;
 }
+*/
 
 // [[Rcpp::export]]
 Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, double par = 0.0, 
-		std::string pattern = "", bool sparse = true, bool prepared = false) {
+		std::string pattern = "", bool prepared = false) {
 
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 
@@ -226,7 +228,7 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 	Rcpp::List ret_list;
 
 	using namespace Rcpp; // so that later on the (i,_) works
-	if (op == "relate") { // character return matrix:
+	if (op == "relate") { // return character matrix:
 		Rcpp::CharacterVector out(sfc0.length() * sfc1.length());
 		for (int i = 0; i < sfc0.length(); i++) {
 			for (int j = 0; j < sfc1.length(); j++) {
@@ -287,9 +289,7 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 			}
 		}
 		ret_list = Rcpp::List::create(out);
-	} else if (op == "is_within_distance") { // return sparse matrix:
-		if (! sparse) // taken care of at R level, use st_distance(x,y) <= z
-			Rcpp::stop("is_within_distance: shouldn't happen"); // #nocov
+	} else if (op == "is_within_distance") {
 		Rcpp::List sparsemat(sfc0.length());
 		for (size_t i = 0; i < gmv0.size(); i++) {
 			std::vector<size_t> sel;
@@ -305,10 +305,7 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 		}
 		ret_list = sparsemat;
 	} else if (gmv1.size()) {
-		// other cases: boolean return matrix, either dense or sparse
-		Rcpp::LogicalMatrix densemat;
-		if (! sparse)  // allocate:
-			densemat = Rcpp::LogicalMatrix(sfc0.length(), sfc1.length());
+		// other cases: sparse matrix
 		Rcpp::List sparsemat(sfc0.length());
 
 		std::vector<size_t> items(gmv1.size());
@@ -324,10 +321,7 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 				Rcpp::LogicalVector rowi(sfc1.length()); 
 				for (int j = 0; j < sfc1.length(); j++) 
 					rowi(j) = chk_(GEOSEqualsExact_r(hGEOSCtxt, gmv0[i], gmv1[j], par));
-				if (! sparse)
-					densemat(i,_) = rowi;
-				else
-					sparsemat[i] = get_which(rowi);
+				sparsemat[i] = get_which(rowi);
 				R_CheckUserInterrupt();
 			}
 		} else if (op == "relate_pattern") { // needing pattern
@@ -342,11 +336,8 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 				for (size_t j = 0; j < tree_sel.size(); j++)
 					if (chk_(GEOSRelatePattern_r(hGEOSCtxt, gmv0[i], gmv1[tree_sel[j]], pattern.c_str())))
 						sel.push_back(tree_sel[j] + 1); // 1-based
-				if (sparse) {
-					std::sort(sel.begin(), sel.end());
-					sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
-				} else // dense
-					densemat(i,_) = get_dense(sel, sfc1.length());
+				std::sort(sel.begin(), sel.end());
+				sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
 				R_CheckUserInterrupt();
 			}
 		} else if (op == "disjoint")
@@ -363,11 +354,8 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 					for (size_t j = 0; j < tree_sel.size(); j++)
 						if (chk_(logical_fn(hGEOSCtxt, pr, gmv1[tree_sel[j]])))
 							sel.push_back(tree_sel[j] + 1); // 1-based
-					if (sparse) {
-						std::sort(sel.begin(), sel.end());
-						sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
-					} else // dense
-						densemat(i,_) = get_dense(sel, sfc1.length());
+					std::sort(sel.begin(), sel.end());
+					sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
 					GEOSPreparedGeom_destroy_r(hGEOSCtxt, pr);
 					R_CheckUserInterrupt();
 				}
@@ -381,32 +369,21 @@ Rcpp::List CPL_geos_binop(Rcpp::List sfc0, Rcpp::List sfc1, std::string op, doub
 					for (size_t j = 0; j < tree_sel.size(); j++)
 						if (chk_(logical_fn(hGEOSCtxt, gmv0[i], gmv1[tree_sel[j]])))
 							sel.push_back(tree_sel[j] + 1); // 1-based
-					if (sparse) {
-						std::sort(sel.begin(), sel.end());
-						sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
-					} else // dense
-						densemat(i,_) = get_dense(sel, sfc1.length());
+					std::sort(sel.begin(), sel.end());
+					sparsemat[i] = Rcpp::IntegerVector(sel.begin(), sel.end());
 					R_CheckUserInterrupt();
 				}
 			}
 		}
 		GEOSSTRtree_destroy_r(hGEOSCtxt, tree1);
-		if (sparse)
-			ret_list = sparsemat;
-		else
-			ret_list = Rcpp::List::create(densemat);
+		ret_list = sparsemat;
 	} else { // gmv1.size() == 0:
-		if (! sparse) { // allocate:
-			Rcpp::LogicalMatrix densemat;
-			densemat = Rcpp::LogicalMatrix(sfc0.length(), sfc1.length());
-			ret_list = Rcpp::List::create(densemat);
-		} else {
-			Rcpp::List sparsemat(sfc0.length());
-			for (size_t i = 0; i < gmv0.size(); i++)
-				sparsemat[i] = Rcpp::IntegerVector();
-			ret_list = sparsemat;
-		}
+		Rcpp::List sparsemat(sfc0.length());
+		for (size_t i = 0; i < gmv0.size(); i++)
+			sparsemat[i] = Rcpp::IntegerVector();
+		ret_list = sparsemat;
 	}
+	// clean up:
 	for (size_t i = 0; i < gmv0.size(); i++)
 		GEOSGeom_destroy_r(hGEOSCtxt, gmv0[i]);
 	for (size_t i = 0; i < gmv1.size(); i++)
@@ -571,9 +548,9 @@ GEOSGeometry *chkNULL(GEOSGeometry *value) {
 
 // [[Rcpp::export]]
 Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc, 
-		Rcpp::NumericVector bufferDist, int nQuadSegs = 30,
-		double dTolerance = 0.0, bool preserveTopology = false, 
-		int bOnlyEdges = 1, double dfMaxLength = 0.0) {
+		Rcpp::NumericVector bufferDist, Rcpp::IntegerVector nQuadSegs,
+		Rcpp::NumericVector dTolerance, Rcpp::LogicalVector preserveTopology, 
+		int bOnlyEdges = 1) {
 
 	int dim = 2;
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init(); 
@@ -585,7 +562,7 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 		if (bufferDist.size() != (int) g.size())
 			Rcpp::stop("invalid dist argument"); // #nocov
 		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSBuffer_r(hGEOSCtxt, g[i], bufferDist[i], nQuadSegs));
+			out[i] = chkNULL(GEOSBuffer_r(hGEOSCtxt, g[i], bufferDist[i], nQuadSegs[i]));
 	} else if (op == "boundary") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = chkNULL(GEOSBoundary_r(hGEOSCtxt, g[i]));
@@ -597,8 +574,9 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 //			out[i] = chkNULL(GEOSUnaryUnion_r(hGEOSCtxt, g[i]));
 	} else if (op == "simplify") {
 		for (size_t i = 0; i < g.size(); i++)
-			out[i] = preserveTopology ? chkNULL(GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i], dTolerance)) :
-					chkNULL(GEOSSimplify_r(hGEOSCtxt, g[i], dTolerance));
+			out[i] = preserveTopology[i] ?
+					chkNULL(GEOSTopologyPreserveSimplify_r(hGEOSCtxt, g[i], dTolerance[i])) :
+					chkNULL(GEOSSimplify_r(hGEOSCtxt, g[i], dTolerance[i]));
 	} else if (op == "linemerge") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = chkNULL(GEOSLineMerge_r(hGEOSCtxt, g[i]));
@@ -621,7 +599,7 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 #if GEOS_VERSION_MAJOR >= 3 && GEOS_VERSION_MINOR >= 4
 	if (op == "triangulate") {
 		for (size_t i = 0; i < g.size(); i++)
-			out[i] = chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i], dTolerance, bOnlyEdges));
+			out[i] = chkNULL(GEOSDelaunayTriangulation_r(hGEOSCtxt, g[i], dTolerance[i], bOnlyEdges));
 	} else
 #endif
 		Rcpp::stop("invalid operation"); // would leak g and out // #nocov
@@ -716,8 +694,8 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 					out.push_back(geom); // keep
 				} else
 					GEOSGeom_destroy_r(hGEOSCtxt, geom); // discard
+				R_CheckUserInterrupt();
 			}
-			R_CheckUserInterrupt();
 		}
 		GEOSSTRtree_destroy_r(hGEOSCtxt, tree);
 
@@ -743,8 +721,8 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 					out.push_back(geom); // keep
 				} else
 					GEOSGeom_destroy_r(hGEOSCtxt, geom); // discard
+				R_CheckUserInterrupt();
 			}
-			R_CheckUserInterrupt();
 		}
 	}
 
@@ -777,12 +755,6 @@ Rcpp::NumericMatrix CPL_geos_dist(Rcpp::List sfc0, Rcpp::List sfc1,
 	return out;
 }
 
-// [[Rcpp::export]]
-Rcpp::CharacterVector CPL_geos_relate(Rcpp::List sfc0, Rcpp::List sfc1) {
-	Rcpp::CharacterVector out = CPL_geos_binop(sfc0, sfc1, "relate", 0.0, "", false)[0];
-	return out;	
-}
-
 // [[Rcpp::export]]
 Rcpp::List CPL_transpose_sparse_incidence(Rcpp::List m, int n) {
 // transpose a sparse incidence matrix list m that has n columns
