@@ -93,7 +93,13 @@ test_that("st_coordinates works", {
 	# expect_true(is.matrix(st_coordinates(st_geometrycollection(list(st_point)))))
 })
 
-test_that("as.data.frame.sfc", {
+test_that("as.data.frame.sfc works", {
   sfc = st_sfc(st_point(0:1), st_point(3:4))
   expect_silent(as.data.frame(sfc))
 })
+
+test_that("rep.sfc works", {
+  expect_equal(
+	st_sfc(st_point(0:1), st_point(0:1), crs = 4326),
+  	rep(st_sfc(st_point(0:1), crs=4326), 2))
+})
