@@ -61,7 +61,7 @@ test_that("delete and update work (#304) ", {
   x <- st_sf(a = 1:2, geom = st_sfc(st_linestring(matrix(1:4,2,2)),
 	st_multilinestring(list(matrix(1:4,2,2), matrix(10:13,2,2)))))
   expect_silent(write_sf(x, shp, "x"))
-  expect_silent(write_sf(x, shp))
+  expect_silent(write_sf(x, shp, delete_dsn = TRUE))
   expect_silent(x <- st_read(shp, quiet = TRUE))
   expect_silent(x <- read_sf(shp))
   expect_error(st_write(x, shp, driver = character(0), quiet = TRUE)) # err
